@@ -8,14 +8,18 @@
 */
 
 /* .Call calls */
+extern SEXP _JMbayes2_cum_haz(SEXP, SEXP);
 extern SEXP _JMbayes2_logLik_jm(SEXP, SEXP, SEXP, SEXP);
 extern SEXP _JMbayes2_mcmc_cpp(SEXP, SEXP, SEXP, SEXP, SEXP);
 extern SEXP _JMbayes2_mlogLik_jm(SEXP, SEXP, SEXP, SEXP, SEXP, SEXP);
+extern SEXP _JMbayes2_simulate_REs(SEXP, SEXP, SEXP);
 
 static const R_CallMethodDef CallEntries[] = {
-    {"_JMbayes2_logLik_jm",  (DL_FUNC) &_JMbayes2_logLik_jm,  4},
-    {"_JMbayes2_mcmc_cpp",   (DL_FUNC) &_JMbayes2_mcmc_cpp,   5},
-    {"_JMbayes2_mlogLik_jm", (DL_FUNC) &_JMbayes2_mlogLik_jm, 6},
+    {"_JMbayes2_cum_haz",      (DL_FUNC) &_JMbayes2_cum_haz,      2},
+    {"_JMbayes2_logLik_jm",    (DL_FUNC) &_JMbayes2_logLik_jm,    4},
+    {"_JMbayes2_mcmc_cpp",     (DL_FUNC) &_JMbayes2_mcmc_cpp,     5},
+    {"_JMbayes2_mlogLik_jm",   (DL_FUNC) &_JMbayes2_mlogLik_jm,   6},
+    {"_JMbayes2_simulate_REs", (DL_FUNC) &_JMbayes2_simulate_REs, 3},
     {NULL, NULL, 0}
 };
 
